# -*- coding: utf-8 -*-

"""
verizon

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from verizon.api_helper import APIHelper
from verizon.configuration import Server
from verizon.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from verizon.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from apimatic_core.authentication.multiple.and_auth_group import And
from verizon.models.configuration_list_item import ConfigurationListItem
from verizon.models.api_v_1_application_configurations_geofence_response import ApiV1ApplicationConfigurationsGeofenceResponse
from verizon.exceptions.api_v_1_application_configurations_geofence_ids_403_error_exception import ApiV1ApplicationConfigurationsGeofenceIds403ErrorException
from verizon.exceptions.api_v_1_application_configurations_geofence_ids_404_error_exception import ApiV1ApplicationConfigurationsGeofenceIds404ErrorException
from verizon.exceptions.api_v_1_application_configurations_geofence_ids_429_error_exception import ApiV1ApplicationConfigurationsGeofenceIds429ErrorException
from verizon.exceptions.api_v_1_application_configurations_geofence_ids_0_error_exception import ApiV1ApplicationConfigurationsGeofenceIds0ErrorException
from verizon.exceptions.api_v_1_application_configurations_geofence_403_error_exception import ApiV1ApplicationConfigurationsGeofence403ErrorException
from verizon.exceptions.api_v_1_application_configurations_geofence_404_error_exception import ApiV1ApplicationConfigurationsGeofence404ErrorException
from verizon.exceptions.api_v_1_application_configurations_geofence_429_error_exception import ApiV1ApplicationConfigurationsGeofence429ErrorException
from verizon.exceptions.api_v_1_application_configurations_geofence_0_error_exception import ApiV1ApplicationConfigurationsGeofence0ErrorException
from verizon.exceptions.api_v_1_application_configurations_geofence_400_error_exception import ApiV1ApplicationConfigurationsGeofence400ErrorException


class AppConfigServiceController(BaseController):

    """A Controller to access Endpoints in the verizon API."""
    def __init__(self, config):
        super(AppConfigServiceController, self).__init__(config)

    def get_configuration_list(self,
                               vendor_id):
        """Does a GET request to /api/v1/application/configurations/geofence/ids.

        This endpoint fetches and returns the list of configurations defined
        by the Vendor. The list contains the configurations' identifier, name,
        description, and active flag. The vendor ID is provided when the
        configuration is created through the POST request.
        Note: The user needs to authenticate with their ThingSpace credentials
        using the Access/Bearer and Session/M2M tokens in order to call this
        API.

        Args:
            vendor_id (str): The vendor's identifier

        Returns:
            List[ConfigurationListItem]: Response from the API. Configuration
                list was queried successfully

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/application/configurations/geofence/ids')
            .http_method(HttpMethodEnum.GET)
            .header_param(Parameter()
                          .key('VendorID')
                          .value(vendor_id))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(And(Single('thingspace_oauth'), Single('SessionToken')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ConfigurationListItem.from_dictionary)
            .local_error('403', 'Forbidden', ApiV1ApplicationConfigurationsGeofenceIds403ErrorException)
            .local_error('404', 'Configuration not found', ApiV1ApplicationConfigurationsGeofenceIds404ErrorException)
            .local_error('429', 'Too many requests', ApiV1ApplicationConfigurationsGeofenceIds429ErrorException)
            .local_error('default', 'unexpected error', ApiV1ApplicationConfigurationsGeofenceIds0ErrorException)
        ).execute()

    def get_configuration(self,
                          id,
                          vendor_id):
        """Does a GET request to /api/v1/application/configurations/geofence.

        This endpoint fetches and returns a specific configuration's details.
        The configuration ID parameter, which was provided when the
        configuration was created through the POST request, is need to
        retrieve the configuration details.
        Note: The user needs to authenticate with their ThingSpace credentials
        using the Access/Bearer and Session/M2M tokens in order to call this
        API.

        Args:
            id (str): The configuration identifier
            vendor_id (str): The vendor's identifier

        Returns:
            ApiV1ApplicationConfigurationsGeofenceResponse: Response from the
                API. Configuration found

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/application/configurations/geofence')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('id')
                         .value(id))
            .header_param(Parameter()
                          .key('VendorID')
                          .value(vendor_id))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(And(Single('thingspace_oauth'), Single('SessionToken')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ApiV1ApplicationConfigurationsGeofenceResponse.from_dictionary)
            .local_error('403', 'Forbidden', ApiV1ApplicationConfigurationsGeofence403ErrorException)
            .local_error('404', 'Configuration not found', ApiV1ApplicationConfigurationsGeofence404ErrorException)
            .local_error('429', 'Too many requests', ApiV1ApplicationConfigurationsGeofence429ErrorException)
            .local_error('default', 'unexpected error', ApiV1ApplicationConfigurationsGeofence0ErrorException)
        ).execute()

    def create_configuration(self,
                             vendor_id,
                             body):
        """Does a POST request to /api/v1/application/configurations/geofence.

        This endpoint creates a new configuration in the system. The data for
        the new configuration should be provided as JSON in the body of the
        POST request. The system will return with a unique ID for the
        configuration, which is needed for any further manipulation (update or
        delete) of the configuration.
        Note: The user needs to authenticate with their ThingSpace credentials
        using the Access/Bearer and Session/M2M tokens in order to call this
        API.

        Args:
            vendor_id (str): The vendor's identifier
            body (ApiV1ApplicationConfigurationsGeofenceRequest): The request
                body parameter.

        Returns:
            ApiV1ApplicationConfigurationsGeofenceResponse: Response from the
                API. Configuration created

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/application/configurations/geofence')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('VendorID')
                          .value(vendor_id))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(And(Single('thingspace_oauth'), Single('SessionToken')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ApiV1ApplicationConfigurationsGeofenceResponse.from_dictionary)
            .local_error('400', 'Invalid configuration', ApiV1ApplicationConfigurationsGeofence400ErrorException)
            .local_error('403', 'Forbidden', ApiV1ApplicationConfigurationsGeofence403ErrorException)
            .local_error('429', 'Too many requests', ApiV1ApplicationConfigurationsGeofence429ErrorException)
            .local_error('default', 'unexpected error', ApiV1ApplicationConfigurationsGeofence0ErrorException)
        ).execute()

    def update_configuration(self,
                             vendor_id,
                             id,
                             body):
        """Does a PUT request to /api/v1/application/configurations/geofence.

        This endpoint updates an existing configuration. Similar to POST, the
        updated data for the configuration should be provided as JSON in the
        body of the PUT request. The configuration ID parameter, which was
        provided by the POST (create) operation, is required to do any updates
        on the configuration.
        Note: The user needs to authenticate with their ThingSpace credentials
        using the Access/Bearer and Session/M2M tokens in order to call this
        API.

        Args:
            vendor_id (str): The vendor's identifier
            id (str): The configuration identifier
            body (ApiV1ApplicationConfigurationsGeofenceRequest1): The request
                body parameter.

        Returns:
            void: Response from the API. Configuration applied

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/application/configurations/geofence')
            .http_method(HttpMethodEnum.PUT)
            .header_param(Parameter()
                          .key('VendorID')
                          .value(vendor_id))
            .query_param(Parameter()
                         .key('id')
                         .value(id))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(And(Single('thingspace_oauth'), Single('SessionToken')))
        ).execute()

    def delete_configuration(self,
                             vendor_id,
                             id):
        """Does a DELETE request to /api/v1/application/configurations/geofence.

        This endpoint deletes a specific configuration from the system. It
        requires the configuration ID parameter, which was provided by the
        POST (create) operation.
        Note: The user needs to authenticate with their ThingSpace credentials
        using the Access/Bearer and Session/M2M tokens in order to call this
        API.

        Args:
            vendor_id (str): The vendor's identifier
            id (str): The configuration identifier

        Returns:
            void: Response from the API. Configuration deleted

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/application/configurations/geofence')
            .http_method(HttpMethodEnum.DELETE)
            .header_param(Parameter()
                          .key('VendorID')
                          .value(vendor_id))
            .query_param(Parameter()
                         .key('id')
                         .value(id))
            .auth(And(Single('thingspace_oauth'), Single('SessionToken')))
        ).execute()
