# -*- coding: utf-8 -*-

"""
verizon

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from verizon.api_helper import APIHelper
from verizon.configuration import Server
from verizon.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from verizon.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from apimatic_core.authentication.multiple.and_auth_group import And
from verizon.exceptions.api_v_2_mapdata_400_error_exception import ApiV2Mapdata400ErrorException
from verizon.exceptions.api_v_2_mapdata_401_error_exception import ApiV2Mapdata401ErrorException
from verizon.exceptions.api_v_2_mapdata_403_error_exception import ApiV2Mapdata403ErrorException
from verizon.exceptions.api_v_2_mapdata_429_error_exception import ApiV2Mapdata429ErrorException
from verizon.exceptions.api_v_2_mapdata_503_error_exception import ApiV2Mapdata503ErrorException
from verizon.exceptions.api_v_2_mapdata_0_error_exception import ApiV2Mapdata0ErrorException


class MapDataManagerController(BaseController):

    """A Controller to access Endpoints in the verizon API."""
    def __init__(self, config):
        super(MapDataManagerController, self).__init__(config)

    def upload_map_data_message(self,
                                vendor_id,
                                body):
        """Does a POST request to /api/v2/mapdata.

        This endpoint allows the user to upload map messages in SAE J2735 MAP
        messages in ASN.1 UPER or JER (JSON) formats. The MAP data message can
        have more than one intersections in it.
        Note: The user needs to authenticate with their ThingSpace credentials
        using the Access/Bearer and Session/M2M tokens in order to call this
        API.

        Args:
            vendor_id (str): The VendorID set during the Vendor registration
                call.
            body (str): The request body parameter.

        Returns:
            str: Response from the API. Map message/s successfully processed

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v2/mapdata')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('VendorID')
                          .value(vendor_id))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .auth(And(Single('thingspace_oauth'), Single('SessionToken')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .local_error('400', 'Invalid request', ApiV2Mapdata400ErrorException)
            .local_error('401', 'Unauthorized', ApiV2Mapdata401ErrorException)
            .local_error('403', 'Forbidden Request', ApiV2Mapdata403ErrorException)
            .local_error('429', 'Too Many Requests', ApiV2Mapdata429ErrorException)
            .local_error('503', 'Internal server Error', ApiV2Mapdata503ErrorException)
            .local_error('default', 'Unexpected Error', ApiV2Mapdata0ErrorException)
        ).execute()

    def download_map_data_message(self,
                                  vendor_id,
                                  geofence):
        """Does a GET request to /api/v2/mapdata.

        This endpoint allows user to download SAE J2735 MAP messages in ASN.1
        UPER format. The area for the MAP messages is needed to be defined in
        the query.

        Args:
            vendor_id (str): The VendorID set during the Vendor registration
                call.
            geofence (Geofence): The area where the user wants to receive MAP
                data from.

        Returns:
            str: Response from the API. The response is a list line separated
                ASN.1 UPER J2735 base64 encoded MapData messages

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v2/mapdata')
            .http_method(HttpMethodEnum.GET)
            .header_param(Parameter()
                          .key('VendorID')
                          .value(vendor_id))
            .query_param(Parameter()
                         .key('Geofence')
                         .value(geofence))
            .auth(And(Single('thingspace_oauth'), Single('SessionToken')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .local_error('400', 'Invalid request', ApiV2Mapdata400ErrorException)
            .local_error('401', 'Unauthorized', ApiV2Mapdata401ErrorException)
            .local_error('403', 'Forbidden Request', ApiV2Mapdata403ErrorException)
            .local_error('429', 'Too Many Requests', ApiV2Mapdata429ErrorException)
            .local_error('503', 'Internal server Error', ApiV2Mapdata503ErrorException)
            .local_error('default', 'Unexpected Error', ApiV2Mapdata0ErrorException)
        ).execute()
