# -*- coding: utf-8 -*-

"""
verizon

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from verizon.api_helper import APIHelper
from verizon.configuration import Server
from verizon.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from verizon.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from apimatic_core.authentication.multiple.and_auth_group import And
from verizon.models.serviceendpoints_response import ServiceendpointsResponse
from verizon.models.serviceendpoints_response_1 import ServiceendpointsResponse1
from verizon.models.serviceendpointsall_response import ServiceendpointsallResponse
from verizon.models.resources_edge_hosted_service_with_profile_id import ResourcesEdgeHostedServiceWithProfileId
from verizon.models.serviceendpoints_response_2 import ServiceendpointsResponse2
from verizon.models.serviceendpoints_response_3 import ServiceendpointsResponse3
from verizon.exceptions.serviceendpoints_400_error_exception import Serviceendpoints400ErrorException
from verizon.exceptions.serviceendpoints_401_error_exception import Serviceendpoints401ErrorException
from verizon.exceptions.serviceendpoints_0_error_exception import Serviceendpoints0ErrorException
from verizon.exceptions.serviceendpointsall_400_error_exception import Serviceendpointsall400ErrorException
from verizon.exceptions.serviceendpointsall_401_error_exception import Serviceendpointsall401ErrorException
from verizon.exceptions.serviceendpointsall_0_error_exception import Serviceendpointsall0ErrorException


class ServiceEndpointsController(BaseController):

    """A Controller to access Endpoints in the verizon API."""
    def __init__(self, config):
        super(ServiceEndpointsController, self).__init__(config)

    def list_optimal_service_endpoints(self,
                                       region=None,
                                       subscriber_density=None,
                                       ue_identity_type=None,
                                       ue_identity=None,
                                       service_endpoints_ids=None):
        """Does a GET request to /serviceendpoints.

        Returns a list of optimal Service Endpoints that client devices can
        connect to. **Note:** If a query is sent with all of the parameters,
        it will fail with a "400" error. You can search based on the following
        parameter combinations - Region plus Service Endpoints IDs and
        Subscriber density (density is optional but recommended), Region plus
        Service Endpoints IDs and UEIdentity(Including UEIdentity Type) and
        Service Endpoints IDs plus UEIdentity(Including UEIdentity Type).

        Args:
            region (str, optional): MEC region name. Current valid values are
                US_WEST_2 and US_EAST_1.
            subscriber_density (int, optional): Minimum number of 4G/5G
                subscribers per square kilometer.
            ue_identity_type (UserEquipmentIdentityTypeEnum, optional): Type
                of User Equipment identifier used in `UEIdentity`.
            ue_identity (str, optional): The identifier value for User
                Equipment. The type of identifier is defined by the
                'UEIdentityType' parameter. The`IPAddress`format can be IPv4
                or IPv6.
            service_endpoints_ids (str, optional): A system-defined string
                identifier representing one or more registered Service
                Endpoints.

        Returns:
            ServiceendpointsResponse: Response from the API. An array of
                optimal Service Endpoint IDs for clients to connect to.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/serviceendpoints')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('region')
                         .value(region))
            .query_param(Parameter()
                         .key('subscriberDensity')
                         .value(subscriber_density))
            .query_param(Parameter()
                         .key('UEIdentityType')
                         .value(ue_identity_type))
            .query_param(Parameter()
                         .key('UEIdentity')
                         .value(ue_identity))
            .query_param(Parameter()
                         .key('serviceEndpointsIds')
                         .value(service_endpoints_ids))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(And(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ServiceendpointsResponse.from_dictionary)
            .local_error('400', 'HTTP 400 Bad Request.', Serviceendpoints400ErrorException)
            .local_error('401', 'HTTP 401 Unauthorized.', Serviceendpoints401ErrorException)
            .local_error('default', 'HTTP 500 Internal Server Error.', Serviceendpoints0ErrorException)
        ).execute()

    def register_service_endpoints(self,
                                   body):
        """Does a POST request to /serviceendpoints.

        Register Service Endpoints of a deployed application to specified MEC
        Platforms.

        Args:
            body (List[ResourcesEdgeHostedServiceWithProfileId]): An array of
                Service Endpoint data for a deployed application. The request
                body passes all of the needed parameters to create a service
                endpoint. Parameters will be edited here rather than the
                **Parameters** section above. The
                `ern`,`applicationServerProviderId`, `applicationId` and
                `serviceProfileID` parameters are required. **Note:**
                Currently, the only valid value for
                `applicationServerProviderId`is **AWS**. Also, if you do not
                know one of the optional values (i.e. URI), you can erase the
                line from the query by back-spacing over it.

        Returns:
            ServiceendpointsResponse1: Response from the API. Returns a
                service endpoints Id.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/serviceendpoints')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(And(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ServiceendpointsResponse1.from_dictionary)
            .local_error('400', 'HTTP 400 Bad Request.', Serviceendpoints400ErrorException)
            .local_error('401', 'HTTP 401 Unauthorized.', Serviceendpoints401ErrorException)
            .local_error('default', 'HTTP 500 Internal Server Error.', Serviceendpoints0ErrorException)
        ).execute()

    def list_all_service_endpoints(self):
        """Does a GET request to /serviceendpointsall.

        Returns a list of all registered service endpoints.

        Returns:
            ServiceendpointsallResponse: Response from the API. Returns a
                comma delimited list of all registered service endpoints.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/serviceendpointsall')
            .http_method(HttpMethodEnum.GET)
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(And(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ServiceendpointsallResponse.from_dictionary)
            .local_error('400', 'HTTP 400 Bad Request.', Serviceendpointsall400ErrorException)
            .local_error('401', 'HTTP 401 Unauthorized.', Serviceendpointsall401ErrorException)
            .local_error('default', 'HTTP 500 Internal Server Error.', Serviceendpointsall0ErrorException)
        ).execute()

    def get_service_endpoint(self,
                             service_endpoints_id):
        """Does a GET request to /serviceendpoints/{serviceEndpointsId}.

        Returns endpoint information for all Service Endpoints registered to a
        specified serviceEndpointId.

        Args:
            service_endpoints_id (str): A system-defined string identifier
                representing one or more registered Service Endpoints.

        Returns:
            List[ResourcesEdgeHostedServiceWithProfileId]: Response from the
                API. Registered service endpoint information for a 5G Edge
                service.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/serviceendpoints/{serviceEndpointsId}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('serviceEndpointsId')
                            .value(service_endpoints_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(And(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ResourcesEdgeHostedServiceWithProfileId.from_dictionary)
            .local_error('400', 'HTTP 400 Bad Request.', Serviceendpoints400ErrorException)
            .local_error('401', 'HTTP 401 Unauthorized.', Serviceendpoints401ErrorException)
            .local_error('default', 'HTTP 500 Internal Server Error.', Serviceendpoints0ErrorException)
        ).execute()

    def update_service_endpoint(self,
                                service_endpoints_id,
                                body):
        """Does a PUT request to /serviceendpoints/{serviceEndpointsId}.

        Update registered Service Endpoint information.

        Args:
            service_endpoints_id (str): A system-defined string identifier
                representing one or more registered Service Endpoints.
            body (List[ResourcesEdgeHostedServiceWithProfileId]): Data needed
                for Service Endpoint information. The request body passes the
                rest of the needed parameters to create a service endpoint.
                Parameters other than `serviceEndpointsId` will be edited here
                rather than the **Parameters** section above. The
                `ern`,`applicationServerProviderId` and `applicationId`
                parameters are required. **Note:** Currently, the only valid
                value for `applicationServerProviderId`is **AWS**.

        Returns:
            ServiceendpointsResponse2: Response from the API. Update
                registered Service Endpoint information.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/serviceendpoints/{serviceEndpointsId}')
            .http_method(HttpMethodEnum.PUT)
            .template_param(Parameter()
                            .key('serviceEndpointsId')
                            .value(service_endpoints_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(And(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ServiceendpointsResponse2.from_dictionary)
            .local_error('400', 'HTTP 400 Bad Request.', Serviceendpoints400ErrorException)
            .local_error('401', 'HTTP 401 Unauthorized.', Serviceendpoints401ErrorException)
            .local_error('default', 'HTTP 500 Internal Server Error.', Serviceendpoints0ErrorException)
        ).execute()

    def deregister_service_endpoint(self,
                                    service_endpoints_id):
        """Does a DELETE request to /serviceendpoints/{serviceEndpointsId}.

        Deregister an application's Service Endpoint from the MEC Platform(s).

        Args:
            service_endpoints_id (str): A system-defined string identifier
                representing one or more registered Service Endpoints.

        Returns:
            ServiceendpointsResponse3: Response from the API. Service endpoint
                deleted.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/serviceendpoints/{serviceEndpointsId}')
            .http_method(HttpMethodEnum.DELETE)
            .template_param(Parameter()
                            .key('serviceEndpointsId')
                            .value(service_endpoints_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(And(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ServiceendpointsResponse3.from_dictionary)
            .local_error('400', 'HTTP 400 Bad Request.', Serviceendpoints400ErrorException)
            .local_error('401', 'HTTP 401 Unauthorized.', Serviceendpoints401ErrorException)
            .local_error('default', 'HTTP 500 Internal Server Error.', Serviceendpoints0ErrorException)
        ).execute()
