# -*- coding: utf-8 -*-

"""
verizon

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from verizon.api_helper import APIHelper
from verizon.configuration import Server
from verizon.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from verizon.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from apimatic_core.authentication.multiple.and_auth_group import And
from verizon.models.serviceprofiles_response import ServiceprofilesResponse
from verizon.models.serviceprofiles_response_1 import ServiceprofilesResponse1
from verizon.models.serviceprofiles_response_2 import ServiceprofilesResponse2
from verizon.models.serviceprofiles_response_3 import ServiceprofilesResponse3
from verizon.models.serviceprofiles_response_4 import ServiceprofilesResponse4
from verizon.exceptions.serviceprofiles_400_error_exception import Serviceprofiles400ErrorException
from verizon.exceptions.serviceprofiles_401_error_exception import Serviceprofiles401ErrorException
from verizon.exceptions.serviceprofiles_0_error_exception import Serviceprofiles0ErrorException


class ServiceProfilesController(BaseController):

    """A Controller to access Endpoints in the verizon API."""
    def __init__(self, config):
        super(ServiceProfilesController, self).__init__(config)

    def create_service_profile(self,
                               body):
        """Does a POST request to /serviceprofiles.

        Creates a service profile that describes the resource requirements of
        a service.

        Args:
            body (ServiceprofilesRequest): The request body passes all of the
                needed parameters to create a service profile. Parameters will
                be edited here rather than the **Parameters** section above.
                The `maxLatencyMs` and `clientType` parameters are both
                required in the request body. **Note:** The `maxLatencyMs`
                value must be submitted in multiples of 5. Additionally, "GPU"
                is future functionality and the values are not captured.

        Returns:
            ServiceprofilesResponse: Response from the API. Service profile ID.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/serviceprofiles')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(And(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ServiceprofilesResponse.from_dictionary)
            .local_error('400', 'HTTP 400 Bad Request.', Serviceprofiles400ErrorException)
            .local_error('401', 'HTTP 401 Unauthorized.', Serviceprofiles401ErrorException)
            .local_error('default', 'HTTP 500 Internal Server Error.', Serviceprofiles0ErrorException)
        ).execute()

    def list_service_profiles(self):
        """Does a GET request to /serviceprofiles.

        List all service profiles registered under your API key.

        Returns:
            ServiceprofilesResponse1: Response from the API. A comma delimited
                list of all the service profiles registered under your API key.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/serviceprofiles')
            .http_method(HttpMethodEnum.GET)
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(And(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ServiceprofilesResponse1.from_dictionary)
            .local_error('400', 'HTTP 400 Bad Request.', Serviceprofiles400ErrorException)
            .local_error('401', 'HTTP 401 Unauthorized.', Serviceprofiles401ErrorException)
            .local_error('default', 'HTTP 500 Internal Server Error.', Serviceprofiles0ErrorException)
        ).execute()

    def get_service_profile(self,
                            service_profile_id):
        """Does a GET request to /serviceprofiles/{serviceProfileId}.

        Returns a specified service profile.

        Args:
            service_profile_id (str): The request template parameter.

        Returns:
            ServiceprofilesResponse2: Response from the API. Requested service
                profile.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/serviceprofiles/{serviceProfileId}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('serviceProfileId')
                            .value(service_profile_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(And(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ServiceprofilesResponse2.from_dictionary)
            .local_error('400', 'HTTP 400 Bad Request.', Serviceprofiles400ErrorException)
            .local_error('401', 'HTTP 401 Unauthorized.', Serviceprofiles401ErrorException)
            .local_error('default', 'HTTP 500 Internal Server Error.', Serviceprofiles0ErrorException)
        ).execute()

    def update_service_profile(self,
                               service_profile_id,
                               body):
        """Does a PUT request to /serviceprofiles/{serviceProfileId}.

        Update the definition of a Service Profile.

        Args:
            service_profile_id (str): The request template parameter.
            body (ServiceprofilesRequest): The request body passes the rest of
                the needed parameters to create a service profile. The
                `maxLatencyMs` and `clientType` parameters are both required
                in the request body. **Note:** The `maxLatencyMs` value must
                be submitted in multiples of 5. Additionally, "GPU" is future
                functionality and the values are not captured. Default values
                to use are shown.

        Returns:
            ServiceprofilesResponse3: Response from the API. Update a service
                profile.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/serviceprofiles/{serviceProfileId}')
            .http_method(HttpMethodEnum.PUT)
            .template_param(Parameter()
                            .key('serviceProfileId')
                            .value(service_profile_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(And(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ServiceprofilesResponse3.from_dictionary)
            .local_error('400', 'HTTP 400 Bad Request.', Serviceprofiles400ErrorException)
            .local_error('401', 'HTTP 401 Unauthorized.', Serviceprofiles401ErrorException)
            .local_error('default', 'HTTP 500 Internal Server Error.', Serviceprofiles0ErrorException)
        ).execute()

    def delete_service_profile(self,
                               service_profile_id):
        """Does a DELETE request to /serviceprofiles/{serviceProfileId}.

        Delete Service Profile based on unique service profile ID.

        Args:
            service_profile_id (str): The request template parameter.

        Returns:
            ServiceprofilesResponse4: Response from the API. Delete a service
                profile.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/serviceprofiles/{serviceProfileId}')
            .http_method(HttpMethodEnum.DELETE)
            .template_param(Parameter()
                            .key('serviceProfileId')
                            .value(service_profile_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(And(Single('thingspace_oauth'), Single('VZ-M2M-Token')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ServiceprofilesResponse4.from_dictionary)
            .local_error('400', 'HTTP 400 Bad Request.', Serviceprofiles400ErrorException)
            .local_error('401', 'HTTP 401 Unauthorized.', Serviceprofiles401ErrorException)
            .local_error('default', 'HTTP 500 Internal Server Error.', Serviceprofiles0ErrorException)
        ).execute()
